#include <dos.h>
#include "trdir.h"


static BYTE dirpool[4096];


/* forces an adapter interrupt and returns information about its state. */
BYTE dir_interrupt (CCB *ccbptr, BYTE adapternum)
{

        ccbptr->command  = DIR_INTERRUPT;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = ccbptr->pointer = ccbptr->cmd_cplt = ZEROADDRESS;

        TRNcmd((unsigned char *) ccbptr);

        return ( ccbptr->retcode );
}


/* Reads the general status information of the adapter */
BYTE dir_status (CCB *ccbptr, BYTE adapternum)
{
        static STATUS_PT pt;

        ccbptr->command  = DIR_STATUS;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        memset((void *) &pt,0,sizeof(STATUS_PT) );

        return (TRNcmd((unsigned char *) ccbptr));
}


/* Initializes the adapter support interface areas, resets all adapter 
   tables and buffers, and directs the adapter to run the bring-up tests. */
BYTE dir_initialize (CCB *ccbptr, BYTE adapternum)
{
        static DIR_INIT_PT pt;

        ccbptr->command  = DIR_INITIALIZE;
        ccbptr->adapter  = adapternum;
        ccbptr->retcode  = 0xFF;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->cmd_cplt = ccbptr->pointer = ZEROADDRESS;

        memset((void *) &pt,0,sizeof(DIR_INIT_PT) );

        return (TRNcmd((unsigned char *) ccbptr));
}



/* opens the adapter so that it is ready for normal ring communication.  
   All buffers & tables will be re-initialized. */
BYTE dir_openadapter (CCB *ccbptr, BYTE adapternum, FNPTR StatApp)
{
        static DIR_OPEN_PT      openpt;
        static DIR_OPEN_AD_PT   adpt;
        static DIR_OPEN_DIR_PT  dirpt;
        static DIR_OPEN_DLC_PT  dlcpt;



        ccbptr->command  = DIR_OPEN_ADAPTER;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = (ADDRESS) &openpt;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        openpt.adapter_parms = (ADDRESS) &adpt;
        openpt.direct_parms  = (ADDRESS) &dirpt;
        openpt.dlc_parms     = (ADDRESS) &dlcpt;
        openpt.msg_parms     = ZEROADDRESS;

        /* initialze to use defaults */
        memset((void *) &adpt, 0,sizeof(DIR_OPEN_AD_PT)  );
        memset((void *) &dirpt,0,sizeof(DIR_OPEN_DIR_PT) );
        memset((void *) &dlcpt,0,sizeof(DIR_OPEN_DLC_PT) );


        /* set open options and functional address */
        adpt.open_options = 0x3880;
        adpt.func_addr[2] = 0x00;
        adpt.func_addr[3] = 0x18;

        /* set direct interface pool: use 4K default */
        dirpt.dir_pool_address = (ADDRESS) &dirpool;
        dirpt.ring_status_exit = (ADDRESS) StatApp;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* sets a programmable timer to expire in some multiple of half-second 
   periods from 0 to 12,000 (100 minutes).  When the specified time
   expires, the command completes with the retcode */
BYTE dir_timer_set (CCB *ccbptr, BYTE adapternum, int secs)
{

        ccbptr->command  = DIR_TIMER_SET;
        ccbptr->adapter  = adapternum;
        if (secs > 6000)
                secs = 6000;

        /* the function wants the # of half-seconds to appear
           in the first two bytes of the parm_tab field.  The
           following does this because of Intel's wonderful
           byte-swapping feature.
        */
        ccbptr->parm_tab = (ADDRESS) ((unsigned long) secs);
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        /* I shouldn't have to do this, but it won't work if I don't */
        ccbptr->retcode = ccbptr->work = 0;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* cancels a timer that was previously initiated by a DIR.SET.TIMER 
   command. */
BYTE dir_timer_cancel (CCB *ccbptr, BYTE adapternum, ADDRESS timerccbptr)
{

        ccbptr->command  = DIR_TIMER_CANCEL;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = timerccbptr;
        ccbptr->cmd_cplt = ccbptr->pointer = ZEROADDRESS;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* enables the adapter to receive MAC frames */
BYTE receive (CCB *ccbptr, BYTE adapternum, WORD stationid,
        FNPTR RecDataApp)
{
        static RECV_PT pt;

        ccbptr->command  = RECEIVE;
        ccbptr->adapter  = adapternum;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;
        ccbptr->pointer  = ZEROADDRESS;

        memset((void *) &pt,0,sizeof(RECV_PT) );

        pt.station_id   = stationid;
        pt.recv_exit    = (ADDRESS) RecDataApp;
        pt.recv_options = 0x20;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* frees the buffers containing the MAC frame back to the direct
   buffer pool */
BYTE buffer_free (CCB *ccbptr, BYTE adapternum, WORD stationid,
        BUFFER_PT * bufferptr)
{
        static BUFFER_FREE_PT pt;

        ccbptr->command  = BUFFER_FREE;
        ccbptr->adapter  = adapternum;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;
        ccbptr->pointer  = ZEROADDRESS;

        memset((void *) &pt,0,sizeof(BUFFER_FREE_PT) );

        pt.station_id   = stationid;
        pt.first_buffer = bufferptr;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* closes the adapter and stops all communications */
BYTE dir_closeadapter (CCB *ccbptr, BYTE adapternum)
{

        ccbptr->command  = DIR_CLOSE_ADAPTER;
        ccbptr->adapter  = adapternum;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->parm_tab = (ADDRESS) ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;
        ccbptr->pointer  = ZEROADDRESS;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* changes the functional address of the station */
BYTE dir_set_func_address(CCB *ccbptr, BYTE adapternum, DWORD func_addr)
{
        ccbptr->command  = DIR_SET_FUNC_ADDRESS;
        ccbptr->adapter  = adapternum;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->parm_tab = (ADDRESS) func_addr;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;
        ccbptr->pointer  = ZEROADDRESS;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* cancels the previously issued receive command */
BYTE receive_cancel (CCB *ccbptr, BYTE adapternum, WORD stationid)
{

        ccbptr->command  = RECEIVE_CANCEL;
        ccbptr->adapter  = adapternum;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->parm_tab = (ADDRESS) MK_FP(0, stationid);
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;
        ccbptr->pointer  = ZEROADDRESS;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* changes the open parameters */
BYTE dir_modify_open_parms (CCB *ccbptr, BYTE adapternum, FNPTR StatApp)
{
        static DIR_MODIFY_OPEN_PT  modifypt;


        ccbptr->command  = DIR_MODIFY_OPEN_PARMS;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = (ADDRESS) &modifypt;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        /* initialze to use defaults */
        memset((void *) &modifypt,0,sizeof(DIR_MODIFY_OPEN_PT) );



        /* set direct interface pool: use 4K default */
        modifypt.dir_pool_address = (ADDRESS) &dirpool;
        modifypt.open_options = 0x3880;
        modifypt.ring_status_exit = (ADDRESS) StatApp;

        return (TRNcmd((unsigned char *) ccbptr));
}


/* sends a MAC frame */
BYTE dir_transmit(CCB *ccbptr, BYTE adapternum, WORD stationid,
        ADDRESS queue_one)
{
        static XMIT_PT pt;

        ccbptr->command  = TRANSMIT_DIR_FRAME;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        memset(&pt,0,sizeof(XMIT_PT));

        pt.station_id   = stationid;
        pt.queue_one = queue_one;


        return (TRNcmd((unsigned char *) ccbptr));
}


/* reads and resets the adapter log */
BYTE dir_read_log(CCB *ccbptr, BYTE adapternum, BYTE *buffer, WORD bufsize)
{
        static DIR_READ_LOG_PT pt;

        ccbptr->command  = DIR_READ_LOG;
        ccbptr->adapter  = adapternum;
        ccbptr->parm_tab = (ADDRESS) &pt;
        ccbptr->pointer  = ZEROADDRESS;
        ccbptr->cmd_cplt = (ADDRESS) ZEROADDRESS;

        memset(&pt,0,sizeof(DIR_READ_LOG_PT));

        pt.log_id       = 2;
        pt.log_buf_length = bufsize;
        pt.log_buf_addr = buffer;


        return (TRNcmd((unsigned char *) ccbptr));
}


