#include <windows.h>     /* Windows include file */
#include <stdlib.h>      /* Miscellaneous function declarations       */
#include <stdio.h>       /* Declarations for standard I/O routines    */
#include <string.h>      /* String function declarations              */
#include <time.h>

#include "trmgr.Ids"
#include "trmgr.Ext"
#include "trfns.h"
#include "trsmt.h"
#include "trmsg.h"


#define MAXLINE                 80	//maximum string size
#define UPDATE_INTERVAL         15	//screen update interval
#define HELPFILE	"TRMGR.HLP"	//help file name


static void ClearListboxes(void );
static int FillListbox( PGPFPARMS pGpfParms, int idListbox, int idInfo,
	int nTabs, int *pTabs );
static void SetItem(HWND hWnd, int id, char *szText);
static void UpdateStatus(void);

/* Global variables */
SHORT idDisplayType = ID_DYNAMIC;	//display play: static or dynamic
enum Status status = NORMAL;		//ring status
int nAdapters;				//number of adapters on the ring
HWND hWnd[3];				//listbox handles
NameType szDetailsAdapter;		//adapter information

EXTERN HWND hwndMainWindow;		//import from Gpf generated code


/* Force clearing of listboxes by posting them a message */
static void ClearListboxes(void )
{
	int i;
	
	for (i = 0; i < 3; i++)
	   {
	   if (hWnd[i])
		PostMessage(hWnd[i], WM_INITDIALOG, 0, 0L);
	   }
}


/* set edit field with text */
static void SetItem(HWND hWnd, int id, char *szText)
{
    HWND hText;

    hText = GetDlgItem(hWnd, id);
    SendMessage(hText, EM_SETSEL, 0, MAKELONG(0, 32767));
    SendMessage(hText, EM_REPLACESEL, 0, (LONG) szText);
}


/* force updating of main window */
static void UpdateStatus(void)
{
    InvalidateRect(hwndMainWindow, NULL, TRUE);
}


/* fill the listbox with information collected from the station management
   module */
static int FillListbox( PGPFPARMS pGpfParms, int idListbox, int idInfo,
	int nTabs, int *pTabs )
{
	int i;
	int n;
	char szString[MAXLINE];
	HWND hListbox;

	/* save handle */
	if (!hWnd[idInfo])
	   hWnd[idInfo] = pGpfParms->hwnd;
	
	hListbox = GetDlgItem(pGpfParms->hwnd, idListbox);
	SendMessage( hListbox, LB_RESETCONTENT, 0, NULL);
	SendMessage( hListbox, LB_SETTABSTOPS, nTabs, (LONG) (LPINT) pTabs);
	n = GetAdapterInfo(idInfo, SA_GETCOUNT, NULL);
	for (i = 0; i < n; i++)
		{
		if (GetAdapterInfo(idInfo, i, szString) >= 0)
		   {
		   SendMessage( hListbox, LB_ADDSTRING, 
			0, (LONG) szString);

		   }
		}
	return n;
};


/* set the menu display option */
void SetOption( PGPFPARMS pGpfParms )
{
  GpfMenuTick(hwndMainWindow,idDisplayType,FALSE);
  idDisplayType = pGpfParms->Command;
  GpfMenuTick(hwndMainWindow,pGpfParms->Command,TRUE);

};


/* display the ring status in the main window */
void PaintMain( PGPFPARMS pGpfParms )
{
  static bFirst = TRUE;
  static char *szStatus[] = {
	"Normal",
	"Beaconing",
	"Recovery",
	"Removed"
	};
  static char szFormat[] = "Number of Adapters: %d\nRing Status: %s";
  static DWORD color[] = {
	RGB(0,          0,      255),
	RGB(255,        0,      0),
	RGB(255,        0,      255),
	RGB(255,        0,      0)
	};


  RECT rect;
  char szMsg[50];

  /* put a check mark on a display menu item */
  if (bFirst)
	{
	bFirst = FALSE;
	GpfMenuTick(hwndMainWindow,idDisplayType,TRUE);
	}
  
  /* draw the text starting at 1/3 the height of the window */
  GetClientRect(pGpfParms->hwnd, &rect);
  rect.top = (rect.bottom - rect.top) / 3 + rect.top;
  wsprintf(szMsg, szFormat, nAdapters, szStatus[status]);
  SetTextColor(pGpfParms->hdc, color[status]);
  DrawText(pGpfParms->hdc, szMsg, -1, &rect, DT_CENTER | DT_VCENTER);

  /* clear the data base if this station has been removed */
  if (status == REMOVED) ResetInfo(SM_CONFIG);

};


/* select an item in the listbox */
void SelectAdapter( PGPFPARMS pGpfParms )
{
	WORD n;
	char szString[MAXLINE];
	HWND hListbox;
	HWND hText;

	hListbox = GetDlgItem(pGpfParms->hwnd, ID_ADAPTERLIST);
	n = (WORD) SendMessage(hListbox, LB_GETCURSEL, 0, 0L);
	if ((n = (WORD) SendMessage(hListbox, LB_GETTEXT, n, (LONG) szString)) != LB_ERR)
		{
		hText = GetDlgItem(pGpfParms->hwnd, ID_ADDRFIELD);
		SendMessage(hText, EM_SETSEL, 0, MAKELONG(0, 32767));
		szString[n] = 0;
		SendMessage(hText, EM_REPLACESEL, 0, (LONG) &szString[n - 12]);
		
		hText = GetDlgItem(pGpfParms->hwnd, ID_ADAPTERNAME);
		SendMessage(hText, EM_SETSEL, 0, MAKELONG(0, 32767));
		*strchr(szString, '\t') = 0;
		SendMessage(hText, EM_REPLACESEL, 0, (LONG) szString);
		}
};


/* fill the listbox with active adapters */
void FillAdapters( PGPFPARMS pGpfParms )
{

    static int tabs[1] = {511};

    nAdapters = FillListbox(pGpfParms, ID_ADAPTERLIST, SM_CONFIG, 1, tabs);
}


/* fill the listbox with soft error entries */
void FillErrors( PGPFPARMS pGpfParms )
{

    static int tabs[3] = {55, 110, 511};
    static char szDesc[] = "Source\tNaun\tTotal Errors";
    HWND hText;

    hText = GetDlgItem(pGpfParms->hwnd, ID_ERRORDESC);
    SendMessage(hText, EM_SETSEL, 0, MAKELONG(0, 32767));
    SendMessage(hText, EM_SETTABSTOPS, 3, (LONG) tabs);
    SendMessage(hText, EM_REPLACESEL, 0, (LONG) szDesc);
    FillListbox(pGpfParms, ID_ERRORLIST, SM_ERROR, 3, tabs);
}


/* fill the listbox with hardware error entries */
void FillFaults( PGPFPARMS pGpfParms )
{
    static int tabs[3] = {100, 155, 511};
    static char szDesc[] = "Date/Time\tSource\tNaun";
    HWND hText;

    hText = GetDlgItem(pGpfParms->hwnd, ID_FAULTDESC);
    SendMessage(hText, EM_SETSEL, 0, MAKELONG(0, 32767));
    SendMessage(hText, EM_SETTABSTOPS, 3, (LONG) tabs);
    SendMessage(hText, EM_REPLACESEL, 0, (LONG) szDesc);
    FillListbox(pGpfParms, ID_FAULTLIST, SM_FAULT, 3, tabs);
}


/* reset the hard error counters */
void ResetFaults( PGPFPARMS pGpfParms )
{
    ResetInfo(SM_FAULT);
    PostMessage(pGpfParms->hwnd, WM_INITDIALOG, 0, 0L);
}


/* reset the soft error counters */
void ResetErrors( PGPFPARMS pGpfParms )
{
    ResetInfo(SM_ERROR);
    PostMessage(pGpfParms->hwnd, WM_INITDIALOG, 0, 0L);
}


/* reset all counters */
void ResetAll( PGPFPARMS pGpfParms )
{

	if (GpfSendMsg(ID_MSGRESET) == IDYES)
	   {
	   ResetInfo(SM_CONFIG);
	   ClearListboxes();
	   if (ResetIt()) 
		GpfSendMsg(ID_MSGFAILED);
	   else
		GpfSendMsg(ID_MSGOK);
	  }

}


/* remove the selected adapter from the LAN */
void RemoveAdapter( PGPFPARMS pGpfParms )
{
	int n;
	char szString[MAXLINE];
	HWND hListbox;
	HWND hText;

	hListbox = GetDlgItem(pGpfParms->hwnd, ID_ADAPTERLIST);
	n = (WORD) SendMessage(hListbox, LB_GETCURSEL, 0, 0L);
	if ((n = (WORD) SendMessage(hListbox, LB_GETTEXT, n, 
		(LONG) szString)) == LB_ERR)
	  {
	  GpfSendMsg(ID_MSGNOSEL);
	  return;
	  }

	if (GpfSendMsg(ID_MSGREMOVE) == IDYES)
	   if (RemoveIt(&szString[n - 12])) 
		GpfSendMsg(ID_MSGFAILED);
	   else
		{
		GpfSendMsg(ID_MSGOK);
		PostMessage(hListbox, WM_INITDIALOG, 0, 0L);
		}
}


/* Main PeekMessage loop to replace the normal GetMessage loop.
   The PeekMessage loop polls TRAGN whenever Windows is idle. */
void PeekMessageLoop(MSG * pMsg)
{
  time_t now;
  time_t update;
  time_t clock;
  BOOL bTsrInstalled;
  BOOL bEmulate;
  BOOL bGo = TRUE;
  enum Status curStatus;
  int n;

   /* read in the adapter symbolic names */
   ReadConfig();

   /* check if TRAGN has been installed: terminate if not */
   bTsrInstalled = TRAGNInstalled();
   if (bTsrInstalled)
	{
	/* activate TRAGN: terminate if operation fails */
	if (ActivateIt(&bEmulate))
	   PostMessage(hwndMainWindow, WM_CLOSE, 2, NULL);
	}
   else
	PostMessage(hwndMainWindow, WM_CLOSE, 1, NULL);


   update = time(&now);
   while (bGo)
	{

	while (!PeekMessage(pMsg,NULL,NULL,NULL,PM_NOREMOVE))
	    {
	    time(&clock);

	    /* if TRAEM, the agent emulator is installed, read a message
		from it each second. Otherwise poll as often as Windows
		allows */
	    if (!bEmulate || (clock != now))
		{
		now = clock;
		if (bTsrInstalled) 
		   {
		   Poll(&curStatus);

		   /* update status display if the ring status changes */
		   if (curStatus != status)
		      {
		      status = curStatus;
		      UpdateStatus();
		      }
		   }
		}

	    if (clock - update >= UPDATE_INTERVAL)
		{
		update = clock;
		if (idDisplayType == ID_DYNAMIC)
		   ClearListboxes();

		n = GetAdapterInfo(SM_CONFIG, SA_GETCOUNT, NULL);
		if (n != nAdapters)
		   {
		   nAdapters = n;
		   UpdateStatus();
		   }
		}
	    }


	/* pass Windows messages to the appropriate window */
	if (GetMessage(pMsg,NULL,NULL,NULL))
	  {
	  if ((!pMsg->hwnd) || (!IsDialogMessage(GetActiveWindow(),pMsg)))
	    {
	    TranslateMessage(pMsg);
	    DispatchMessage(pMsg);
	    }

	  }
	else
	    bGo = FALSE;

	}

  /* disable TRAGN */
  if (bTsrInstalled)
	InactivateIt();

  /* save the adapter symbolic names */
  SaveConfig();
}


/* display the appropriate message before terminating TRMGR */
void QuitTRMGR( PGPFPARMS pGpfParms )
{
   switch (pGpfParms->wParam)
	{
	case 0:
	   /* solicit confirmation from user before termination */
	   switch (GpfSendMsg(ID_MSGQUIT)){
	      case IDYES:
	    	break;
	      case IDNO:
	    	pGpfParms->mresult = (LONG) TRUE;
	    	break;
	      default:
	    	break;
	   }
	   break;
	case 1:
	   /* the message is displayed if TRMGR finds that TRTSR has not been
	      loaded */
	   GpfSendMsg(ID_MSGNOTSR);
	   break;
	case 2:
	   /* the message is displayed if TRMGR cannot initialize 
		the adapter */
	   GpfSendMsg(ID_MSGINITFAILED);
	   break;
	default:
	   break;
	}



}

/* Change/set the symbolic name of an adapter */
void ChangeName( PGPFPARMS pGpfParms )
{
    char szAdapter[MAXLINE];
    char szName[MAXLINE];

    if (GetDlgItemText(pGpfParms->hwnd, ID_ADDRFIELD, szAdapter, MAXLINE))
	{
	if (GetDlgItemText(pGpfParms->hwnd, ID_ADAPTERNAME, szName, MAXLINE))
	   if (ChangeAdapterName(szAdapter, szName) >= 0)
		PostMessage(pGpfParms->hwnd, WM_INITDIALOG, 0, 0L);
	}
}


/* get the unique adapter address as the key to retrieve detailed 
   information for display */
void RecordAdapter( PGPFPARMS pGpfParms, int id )
{
    int n;
    HWND hListbox;
    HWND hText;
    char szString[MAXLINE];

    hListbox = GetDlgItem(pGpfParms->hwnd, id);
    n = (WORD) SendMessage(hListbox, LB_GETCURSEL, 0, 0L);
    if ((n = (WORD) SendMessage(hListbox, LB_GETTEXT, n, (LONG) szString)) != LB_ERR)
	  strcpy(szDetailsAdapter, &szString[n - 12]);

}


/* display the detailed information of the selected item */
void ShowDetails( PGPFPARMS pGpfParms )
{
    int i;
    AdapterDetails details;
    char szNumber[MAXLINE];


    if (GetAdapterDetails(szDetailsAdapter, &details) < 0)
	{
	GpfSendMsg(ID_MSGFAILED);
	return;
	}

    SetItem(pGpfParms->hwnd, ID_THISADDRV, details.szAdapterAddress);
    SetItem(pGpfParms->hwnd, ID_THISNAMEV, details.szAdapterName);
    SetItem(pGpfParms->hwnd, ID_NAUNADDRV, details.szNaunAddress);
    SetItem(pGpfParms->hwnd, ID_NAUNNAMEV, details.szNaunName);
    SetItem(pGpfParms->hwnd, ID_FAULTADDRV, details.szFaultNaunAddress);
    SetItem(pGpfParms->hwnd, ID_FAULTNAMEV, details.szFaultNaunName);
    SetItem(pGpfParms->hwnd, ID_ERRORADDRV, details.szErrorNaunAddress);
    SetItem(pGpfParms->hwnd, ID_ERRORNAMEV, details.szErrorNaunName);

    if (strlen(details.szErrorNaunAddress))
	SetItem(pGpfParms->hwnd, ID_ERRORTIMEV, _ctime(&details.ErrorTime));
    else
	SetItem(pGpfParms->hwnd, ID_ERRORTIMEV, "");
	
    if (strlen(details.szFaultNaunAddress))
	{
	SetItem(pGpfParms->hwnd, ID_FAULTTIMEV, _ctime(&details.FaultTime));
	SetItem(pGpfParms->hwnd, ID_BEACONTYPEV, itoa(details.nBeaconType, szNumber, 10));
	}
    else
	{
	SetItem(pGpfParms->hwnd, ID_FAULTTIMEV, "");
	SetItem(pGpfParms->hwnd, ID_BEACONTYPEV, "");
	}

    for (i = 0; i < 10; i++)
	SetItem(pGpfParms->hwnd, ID_ERROR1V + i, itoa(details.nErrorCount[i], szNumber, 10));

}


/* derive the full path name of the help file from the path of the
   executable */
void MakeHelpPathName(HANDLE hInstance, char *szFileName)
{
   char *  pcFileName;
   int     nFileNameLen;

   nFileNameLen = GetModuleFileName(hInstance,szFileName,EXE_NAME_MAX_SIZE);
   pcFileName = szFileName + nFileNameLen;

   while (pcFileName > szFileName) {
       if (*pcFileName == '\\' || *pcFileName == ':') {
	   *(++pcFileName) = '\0';
	   break;
       }
   nFileNameLen--;
   pcFileName--;
   }

   if ((nFileNameLen+13) < EXE_NAME_MAX_SIZE) {
       lstrcat(szFileName, HELPFILE);
   }

   else {
       lstrcat(szFileName, "?");
   }

   return;
}
