#include <dos.h>
#include <stdio.h>
#include <dpmi.h>

#include "trmsg.h"
#define __DPMI

#ifdef __DPMI

typedef struct {
        unsigned long edi, esi, ebp, reserved, ebx, edx, ecx, eax;
        unsigned flags, es, ds, fs, gs, ip, cs, sp, ss;
        } RMODE_CALL;

/* This is the interface between TRMGR and TRAGN. TRMGR comunicates with 
   TRAGN by exchanges messages */
int TRMsg(Msgs *msg)
{

        RMODE_CALL regs;
        static Msgs *data = NULL;

        memset(&regs, 0, sizeof(RMODE_CALL));
        regs.eax = msg->msg & 0xffff;
        if (data)
           memcpy((void *)data->adapter, (void *) msg->adapter, 6);

	/* we need to use DPMI because we are in protected mode */
        dpmi_SimRealModeInterrupt( SERVINT, 0, 0, &regs);

        if (!data)
	   /* change real mode segment:offset into selector: offset */
           data = MK_FP(dpmi_SegToDescriptor(regs.edx & 0xffff),
           	regs.eax & 0xffff);


        *msg = *data;

	if (msg->msg == MSG_REQUEST_INACTIVATE)
	   {
	   dpmi_FreeDescriptor(FP_SEG(data));
	   data = NULL;
	   }

        return 0;
}

#else

/* The following are for debugging only */
static char *typename[] =
        {"HOUSEKEEPING", "ERROR", "INDICATION", "BEACON", "CONFIG",
         "REQSTAT", "REQMSG", "REQRESET", "REQREMOVE", "ACTIVATE", "INAVTIVATE"};

static PrintAddr(BYTE *addr)
{
        int i;

        for( i=0; i<6; ++i)
                printf( "%02X", addr[i] );
}


int TRMsg(Msgs *msg)
{

        union REGS regs;
        static Msgs *data;
        
        regs.x.ax = msg->msg;
        if (data)
           memcpy((void *)data->adapter, (void *) msg->adapter, 6);

        int86( SERVINT, &regs, &regs);
        if (data == NULL)
           {
           data = MK_FP(regs.x.dx, regs.x.ax);
           }
        *msg = *data;

        return 0;

}


void TRPrintMsg(Msgs *msg)
{
        int i;

        if (msg->msg > MAXMSGS)
          {
          printf("Invalid Message number: %d\n", msg->msg);
          return;
          }
        else if (msg->retcode == ERR_NO_DATA)
          {
          printf(".");
          return;
          }

        printf("\nMsg type %s:\n", typename[msg->msg]);
        printf("  More msg: %d return code: %d dir return code: %d\n", 
               msg->more, msg->retcode, msg->dir_retcode);
        printf("  Adapter: "); PrintAddr(msg->adapter);
        printf("\n  Naun: "); PrintAddr(msg->naun);
        printf("\n  Trman: "); PrintAddr(msg->type.status.trman_addr);
        printf("\n  Ring Status: %x", msg->type.status.ringstatus);
        printf("\n  Isolating Error: ");
        for (i = 0; i < 5; i++)
           printf("%02x ", msg->type.errcnt.isoerr[i]);
        printf("\n  Non-isolating Error: ");
        for (i = 0; i < 5; i++)
           printf("%02x ", msg->type.errcnt.noniso[i]);
        printf("\n\n");

}
        
#endif
